/*
 *	 Title:	Test fr die Library RTC "DS 12C887"
 *	Author: Andreas Hummel (a.hummel@neblis.de)
 *
 *	Hardware:
 *	Getestet auf ATmega16 und mit RTC "ODIN OEC12C887"
 *
 *	Beschreibung:
 *	Schreibt ein paar allgemeine Daten ber die Einstellungen der RTC auf den UART.
 *	Anschlieend wird per Interrupt INT0, der von der RTC ausgelst wird, jede Sekunde 
 *  das aktuelle Datum und Uhrzeit auf den UART geschrieben.
 *
 *  Siehe auch beiliegendem Anschluschema "connect.gif".
 */

#include <stdlib.h>
#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/signal.h>
#include <avr/delay.h>
#include <avr/sleep.h>
#include "uart.h"
#include "rtc.h"

// Initialiserung der Libraries aufrufen.
void ioinit(void)
{	
	sei();
	
	rtc_init();
	uart_init( UART_BAUD_SELECT(19200,F_CPU) );
}

// Interrupt INT0 aktivieren.
void init_clock( void )
{
	cli();
	PORTD |= _BV( PD2 ); // Pull-UP fr INT0 hoch...
	DDRD &= ~_BV( PD2 ); // PD2 als INPUT
	
	// INT0 einschalten...
	MCUCR &= ~_BV( ISC00 ); // High->Low-Wechsel lst Interrupt aus.
	MCUCR |= _BV( ISC01 ); 	
	GICR  |= _BV( INT0 );	// INT0 aktivieren.
	sei();
}

// Zahl als dezimalen String ausgeben.
void PrintDataDEC( unsigned char c )
{
	char buff[4];
	itoa(c,buff,10);
	uart_puts( buff );
}

// Zahl als binren String ausgeben.
void PrintDataBIN( unsigned char c )
{
	char buff[9];
	itoa(c,buff,2);
	uart_puts( buff );
}

// Schreibt Datum und Uhrzeit neu auf den UART.
void refresh_display( void )
{
	char buffer[11];
	
	rtc_get_day_of_week( buffer ); // Wochentag
	uart_puts( buffer );
	uart_puts_P(", ");
	rtc_get_date( buffer );		// Datum
	uart_puts( buffer );
	uart_puts_P(" ");
	rtc_get_time( buffer );		// Uhrzeit
	uart_puts( buffer );
	
	uart_puts_P("\r");
}

// Wird jede Sekunde von der RTC ausgelst.
SIGNAL( SIG_INTERRUPT0 )
{
	rtc_read_byte( RTC_A_REG_C );	// Interrupt clearen
	refresh_display();
}

// Allgemeine Infos aus der RTC auslesen.
void set_and_print_params( void )
{
	unsigned char regb_soll = _BV( RTC_BIT_B_DMODE ) | _BV( RTC_BIT_B_2412 ) | _BV( RTC_BIT_B_DSE ) | _BV( RTC_BIT_B_AIE );
	unsigned char regb_ist = rtc_read_byte( RTC_A_REG_B );
	
	uart_puts_P("\r\n");
	uart_puts_P( "\r\nREG A: " );
	PrintDataBIN( rtc_read_byte( RTC_A_REG_A ) );
	uart_puts_P( "\r\nREG B: " );
	PrintDataBIN( rtc_read_byte( RTC_A_REG_B ) );
	uart_puts_P( "\r\nREG C: " );
	PrintDataBIN( rtc_read_byte( RTC_A_REG_C ) );
	uart_puts_P( "\r\nREG D: " );
	PrintDataBIN( rtc_read_byte( RTC_A_REG_D ) );
	
	uart_puts_P( "\r\nAlarmSEC: " );
	PrintDataBIN( rtc_read_byte( RTC_A_SEC_ALARM ) );
	uart_puts_P( "\r\nAlarmMIN: " );
	PrintDataBIN( rtc_read_byte( RTC_A_MIN_ALARM ) );
	uart_puts_P( "\r\nAlarmHOUR: " );
	PrintDataBIN( rtc_read_byte( RTC_A_HOUR_ALARM ) );
	
	if( rtc_check_battery() )
		uart_puts_P( "\r\nBatterie OK" );
	else
		uart_puts_P( "\r\nACHTUNG: Batterie Leer!!" );
	
	uart_puts_P("\r\n");
	
	if( regb_ist != regb_soll )
	{
		rtc_write_byte( RTC_A_REG_B, regb_soll );
		uart_puts_P( "Register B neu geschrieben...\r\n" );
	}
}

int main(void)
{
	ioinit();

	rtc_write_byte( RTC_A_SEC_ALARM, 0XC0 );	// Alle Alarm-Bytes auf "don't care" setzen, 
	rtc_write_byte( RTC_A_MIN_ALARM, 0XC0 );	// somit IRQ im Sekundentakt
	rtc_write_byte( RTC_A_HOUR_ALARM, 0XC0 );
	
	set_and_print_params();
	init_clock();

	for(;;)
	{
	}
	
    return (0);
}
