/*
 *	 Title:	Library fr RTC "DS 12C887" oder kompatible
 *	Author: Andreas Hummel (a.hummel@neblis.de)
 *
 *	Hardware:
 *	Getestet auf ATmega16 und mit RTC "ODIN OEC12C887"
 *
 *	Beschreibung:
 *	Einfache Lib, mit der 12C887 RTC's angesprochen werden knnen.
 *	Die definitionen welcher PIN der RTC an welchem AVR-Pin hngt, werden in der
 *  rtc.h-Datei festgelegt.
 *
 *	Siehe auch beiliegendem Anschluschema "connect.gif".
 */

// MCU-Frequenz fr delay.h
//#define F_CPU 4000000 // bei mir in Makefile!
#include <avr/delay.h>

#include <avr/io.h>
#include <string.h>
#include "rtc.h"

// Wait-Makro
#define wait() _delay_us(1.0)

// Initialisiert die PORTs an denen die RTC hngt.
void rtc_init( void )
{
	// Diese Ports auf Output
	RTC_DDR_C = _BV(RTC_DS) | _BV(RTC_RW) | _BV(RTC_AS) | _BV(RTC_CS);
	RTC_DDR_A = 0xFF;
	
	// Alles LOW auf PortA und PortC
	RTC_ADDR = 0;
	RTC_CTRL = 0;	
	
	// Nach PowerUP evtl. warten...
	_delay_ms(200.0);							
}

// Schreibt ein Byte auf die RTC an die angegebene Adresse.
void rtc_write_byte( unsigned char address, unsigned char data )
{
	RTC_DDR_A = 0xFF;								// PortA auf Output
	
	RTC_CTRL |= _BV(RTC_CS);						// CS High
	wait();
	RTC_CTRL |= _BV(RTC_DS) | _BV(RTC_RW);			// DS & RW High
	wait();
	RTC_CTRL |= _BV(RTC_AS);						// AS High
	wait();
	RTC_ADDR = address;								// Addresse Selecten
	wait();								
	RTC_CTRL &= ~_BV(RTC_CS);						// CS Low
	wait();
	RTC_CTRL &= ~_BV(RTC_AS);						// AS Low
	wait();
	RTC_CTRL &= ~_BV(RTC_RW);						// RW Low
	wait();
	RTC_ADDR = data;								// Daten bereitstellen	
	RTC_CTRL |= _BV(RTC_RW);						// RW Up!
	wait();
	RTC_CTRL |= _BV(RTC_CS);						// CS High
	wait();
	RTC_CTRL |= _BV(RTC_AS);						// AS High
	_delay_us(20.0);
}

// Liest ein Byte aus der RTC an der angegebenen Adresse.
unsigned char rtc_read_byte( unsigned char address )
{
	unsigned char data = 0;
	
	RTC_DDR_A = 0xFF;								// PortA auf Output
	
	RTC_CTRL |= _BV(RTC_CS);						// CS High
	wait();
	RTC_CTRL |= _BV(RTC_DS) | _BV(RTC_RW);			// DS & RW High
	wait();
	RTC_CTRL |= _BV(RTC_AS);						// AS High
	wait();
	RTC_ADDR = address;								// Addresse Selecten
	wait();								
	RTC_CTRL &= ~_BV(RTC_CS);						// CS Low
	wait();
	RTC_CTRL &= ~_BV(RTC_AS);						// AS Low
	wait();
	RTC_CTRL &= ~_BV(RTC_DS);						// DS Low
	wait();
	RTC_DDR_A = 0x00;								// PortA auf Input	
	RTC_ADDR = 0xFF;								// Clearen (Pull-Up's Hoch)
	wait();
	data = RTC_DATA;								// Daten lesen...
	wait();
	RTC_CTRL |= _BV(RTC_DS);						// DS Up!
	wait();
	RTC_CTRL |= _BV(RTC_CS);						// CS High
	wait();
	RTC_CTRL |= _BV(RTC_AS);						// AS High
	_delay_us(20.0);
	
	return data;
}

// Gibt eine deutsch formatierte Uhrzeit zurck. Buffer muss mindestens 9 Zeichen lang sein.
char* rtc_get_time( char* buffer )
{
	// Werte aus der RTC lesen...
	unsigned char hour = 0;
	unsigned char min = 0;
	unsigned char sec = 0;
	hour = rtc_read_byte( RTC_A_HOUR );
	min = rtc_read_byte( RTC_A_MIN );
	sec = rtc_read_byte( RTC_A_SEC );

	// Ausgabe im Format: hh:mm:ss
	buffer[0] = '0'+(hour/10);
	buffer[1] = '0'+(hour%10);
	buffer[2] = ':';
	buffer[3] = '0'+(min/10);
	buffer[4] = '0'+(min%10);
	buffer[5] = ':';
	buffer[6] = '0'+(sec/10);
	buffer[7] = '0'+(sec%10);
	buffer[8] = '\0';
	return buffer;
}

// Gibt eine deutsch formatiertes Datum zurck. Buffer muss mindestens 11 Zeichen lang sein.
char* rtc_get_date( char* buffer )
{
	// Werte aus der RTC lesen...
	unsigned char day = 0;
	unsigned char month = 0;
	unsigned char year = 0;
	day = rtc_read_byte( RTC_A_DAY );
	month = rtc_read_byte( RTC_A_MONTH );
	year = rtc_read_byte( RTC_A_YEAR );	
	
	// Ausgabe im Format: dd.mm.yyyy
	buffer[0] = '0'+(day/10);
	buffer[1] = '0'+(day%10);
	buffer[2] = '.';
	buffer[3] = '0'+(month/10);
	buffer[4] = '0'+(month%10);
	buffer[5] = '.';
	buffer[6] = '2';
	buffer[7] = '0';
	buffer[8] = '0'+(year/10);
	buffer[9] = '0'+(year%10);
	buffer[10] = '\0';
	return buffer;
}

// Gibt den deutschen Wochentag zurck. Buffer muss mindestens 3 Zeichen lang sein.
char* rtc_get_day_of_week( char* buffer )
{
	unsigned char day_of_week = 0;
	day_of_week = rtc_read_byte( RTC_A_DAY_OF_WEEK );
	
	switch( day_of_week )
	{
		case 1:
			strcpy( buffer, "So" );
			break;
		case 2:
			strcpy( buffer, "Mo" );
			break;
		case 3:
			strcpy( buffer, "Di" );
			break;
		case 4:
			strcpy( buffer, "Mi" );
			break;
		case 5:
			strcpy( buffer, "Do" );
			break;
		case 6:
			strcpy( buffer, "Fr" );
			break;
		case 7:
			strcpy( buffer, "Sa" );
			break;
	}
	return buffer;
}

// Gibt "TRUE" zurck, falls die Lithiumbatterie in dem RTC in Ordnung ist, sonst "FALSE"
unsigned char	rtc_check_battery( void )
{
	// Solange das MSB im REGISTER D gesetzt ist, ist die Batterie in Ordnung.
	return ( rtc_read_byte( RTC_A_REG_D ) & _BV(7) ) == _BV(7);
}